﻿using Dynamicweb.Core;
using Dynamicweb.Ecommerce.LiveIntegration.Logging;
using Dynamicweb.Environment;

namespace Dynamicweb.Ecommerce.LiveIntegration.Configuration
{
  /// <summary>
  /// Global class with global settings.
  /// </summary>
  internal static class Global
  {
    /// <summary>
    /// Gets a value that determines if the integration is active.
    /// </summary>
    /// <value>Returns<c>true</c> when live integration is enabled and the executing context is not the backend, <c>false</c> otherwise.</value>
    public static bool IsIntegrationActive
    {
      get
      {
        if (!Settings.Instance.IsLiveIntegrationEnabled)
        {
          return false;
        }

        if (ExecutingContext.IsBackEnd())
        {
          return false;
        }

        return true;
      }
    }

    /// <summary>
    /// Gets a value that determines whether integration is enabled for the current shop.
    /// </summary>
    /// <value>Returns<c>true</c> when live integration is enabled for the current shop, <c>false</c> otherwise.</value>
    public static bool IsIntegrationEnabledForShop
    {
      get
      {
        string siteShopId = string.Empty;
        var pv = Dynamicweb.Frontend.PageView.Current();

        if (!string.IsNullOrEmpty(pv?.Area?.EcomShopId))
        {
          siteShopId = pv.Area.EcomShopId;
        }
        else
        {
          Logger.Instance.Log(ErrorLevel.Error, "AreaEcomShopId not found in Area settings");
        }

        return IntegrationEnabledFor(siteShopId);
      }
    }

    /// <summary>
    /// Gets a value that determines if product information should be lazy loaded.
    /// </summary>
    /// <value>Returns <c>true</c> if lazy loaded is enabled in the settings and the current request is not a lazy loaded request, <c>false</c> otherwise.</value>
    public static bool IsProductLazyLoad => Settings.Instance.LazyLoadProductInfo && !Converter.ToBoolean(Context.Current.Request["getproductinfo"]);    

    /// <summary>
    /// Gets a value that determines whether cart communication is enabled.
    /// </summary>
    /// <param name="orderComplete">Indicates if the current order is completed.</param>
    /// <returns>Returns <c>true</c> when the cart communication type is full or when it's "only on order complete" and the current order is complete, <c>false</c> otherwise.</returns>
    public static bool EnableCartCommunication(bool orderComplete)
    {
      string enableCartCommunication = Settings.Instance.CartCommunicationType;

      switch (enableCartCommunication)
      {
        case Constants.CartCommunicationType.None:
          return false;

        case Constants.CartCommunicationType.Full:
          return true;

        case Constants.CartCommunicationType.OnlyOnOrderComplete:
          return orderComplete;

        default:
          return false;
      }
    }

    /// <summary>
    /// Gets a value that determines if the integration is enabled for the specified shop.
    /// </summary>
    /// <param name="shopId">The ID of the shop to check.</param>
    /// <returns>Returns <c>true</c> if integration is enabled for the shop, <c>false</c> otherwise.</returns>
    public static bool IntegrationEnabledFor(string shopId)
    {
      string connectorShopId = Settings.Instance.ShopId;

      return (string.IsNullOrWhiteSpace(connectorShopId) || connectorShopId == shopId) && Connector.IsWebServiceConnectionAvailable();
    }
  }
}