﻿using Dynamicweb.Core;
using Dynamicweb.Ecommerce.International;
using Dynamicweb.Ecommerce.LiveIntegration.Configuration;
using Dynamicweb.Ecommerce.LiveIntegration.Logging;
using Dynamicweb.Ecommerce.Orders;
using Dynamicweb.Environment;
using Dynamicweb.Security.UserManagement;
using System;
using System.Globalization;
using System.Xml;

namespace Dynamicweb.Ecommerce.LiveIntegration
{
    /// <summary>
    /// Class Helpers.
    /// </summary>
    internal static class Helpers
    {
        /// <summary>
        /// Gets the current currency.
        /// </summary>
        /// <value>The current currency.</value>
        public static Currency CurrentCurrency
        {
            get
            {
                Currency ret = Common.Context.Currency;
                if (!string.IsNullOrEmpty(Context.Current.Request.GetString("CurrencyCode")))
                {
                    ret = Common.Application.get_Currency(Context.Current.Request.GetString("CurrencyCode"));
                }
                else if (!Common.Context.CurrencySetByCustomer && Dynamicweb.Frontend.PageView.Current() != null && Dynamicweb.Frontend.PageView.Current().Area != null && !string.IsNullOrEmpty(Convert.ToString(Dynamicweb.Frontend.PageView.Current().Area.EcomCurrencyId)))
                {
                    ret = Common.Application.get_Currency(Convert.ToString(Dynamicweb.Frontend.PageView.Current().Area.EcomCurrencyId));
                }

                return ret;
            }
        }        

        /// <summary>
        /// Adds the child XML node to cart.
        /// </summary>
        /// <param name="xDoc">The x document.</param>
        /// <param name="parent">The parent.</param>
        /// <param name="nodeName">Name of the node.</param>
        /// <param name="nodeValue">The node value.</param>
        public static void AddChildXmlNodeToCart(XmlDocument xDoc, XmlElement parent, string nodeName, string nodeValue)
        {
            XmlElement node = xDoc.CreateElement("column");
            node.SetAttribute("columnName", nodeName);
            node.InnerText = nodeValue;
            parent.AppendChild(node);
        }

        /// <summary>
        /// Adds the child XML node to product list.
        /// </summary>
        /// <param name="xDoc">The x document.</param>
        /// <param name="parent">The parent.</param>
        /// <param name="nodeName">Name of the node.</param>
        /// <param name="nodeValue">The node value.</param>
        public static void AddChildXmlNodeToProductList(XmlDocument xDoc, XmlElement parent, string nodeName, string nodeValue)
        {
            XmlElement node = xDoc.CreateElement(nodeName);
            node.InnerText = nodeValue;
            parent.AppendChild(node);
        }

        /// <summary>
        /// Gets the customer identifier.
        /// </summary>
        /// <param name="user">The user.</param>
        /// <returns>System.String.</returns>
        public static string GetCustomerIdentifier(User user)
        {
            if (user != null)
            {                
                return user.ExternalID;                
            }
            else
            {
                return Settings.Instance.AnonymousUserKey;
            }
        }

        /// <summary>
        /// Returns the relevant Enum Value. If input can be parsed to Enum type, then the enum value is returned. Otherwise defaultValue is returned.
        /// </summary>
        /// <typeparam name="T">Type of Enum to parse value from</typeparam>
        /// <param name="input">To be parsed as Enum value.</param>
        /// <param name="defaultValue">Default value if input was not valid enum value.</param>
        /// <returns>Parsed or default enum value</returns>
        public static T GetEnumValueFromString<T>(string input, T defaultValue)
        {
            if (string.IsNullOrEmpty(input))
            {
                return defaultValue;
            }

            T value;
            try
            {
                value = (T)Enum.Parse(typeof(T), input, true);
            }
            catch (Exception)
            {
                return defaultValue;
            }

            return value;
        }

        /// <summary>
        /// Compares two prices.
        /// </summary>
        /// <param name="price1">The price to compare.</param>
        /// <param name="price2">The price to compare with.</param>
        /// <returns></returns>
        public static bool IsDifferentPrice(double price1, double price2)
        {
            return IsDifferent(price1, price2);
        }

        /// <summary>
        /// Compares two stock levels.
        /// </summary>
        /// <param name="stock1">The stock to compare.</param>
        /// <param name="stock2">The stock to compare with.</param>
        public static bool IsDifferentStock(double stock1, double stock2)
        {
            return IsDifferent(stock1, stock2);
        }

        /// <summary>
        /// Orders the identifier.
        /// </summary>
        /// <param name="order">The order.</param>
        /// <returns>System.String.</returns>
        public static string OrderIdentifier(Order order)
        {
            string ret = order.Id;

            foreach (OrderLine ol in order.OrderLines)
            {
                ret += $".{ol.Quantity}.{ol.ProductId}.{ol.ProductVariantId}.{ol.ProductNumber}.{ol.ProductName}.{ol.Type}";
            }

            return ret;
        }

        /// <summary>
        /// Parses the response to XML.
        /// </summary>
        /// <param name="response">The response.</param>
        /// <param name="responseDoc">The response document.</param>
        /// <returns><c>true</c> if response was parsed to xml, <c>false</c> otherwise.</returns>
        public static bool ParseResponseToXml(string response, out XmlDocument responseDoc)
        {
            bool result = true;
            responseDoc = new XmlDocument();

            try
            {
                responseDoc.LoadXml(response);
            }
            catch (Exception e)
            {
                result = false;
                Logger.Instance.Log(ErrorLevel.ResponseError, $"Response is not valid XML format: '{e.Message}'.");
            }

            return result;
        }

        /// <summary>
        /// Converts the specified value to a double.
        /// </summary>
        /// <param name="value">The value.</param>
        /// <returns>System.Double.</returns>
        public static double ToDouble(string value)
        {
            double result;
            CultureInfo cultureInfo = GetCultureInfo();
            if (cultureInfo != null)
            {
                try
                {
                    result = double.Parse(value, cultureInfo);
                }
                catch (FormatException ex)
                {
                    Logger.Instance.Log(ErrorLevel.Error, $"Error parse '{value}' to double in '{cultureInfo.Name}' culture: '{ex.Message}'.");
                    result = Converter.ToDouble(value);
                }
            }
            else
            {
                result = Converter.ToDouble(value);
            }

            return result;
        }

        /// <summary>
        /// Gets the culture information.
        /// </summary>
        /// <returns>CultureInfo.</returns>
        internal static CultureInfo GetCultureInfo()
        {
            CultureInfo result = null;

            string culture = Settings.Instance.NumberFormatCulture;
            if (!string.IsNullOrEmpty(culture))
            {
                result = GetCultureInfo(culture, true);
            }
            else
            {
                result = CultureInfo.InvariantCulture;
            }

            return result;
        }

        private static bool IsDifferent(double x, double y)
        {
            var tolerance = 0.0000001;
            var isDifferent = Math.Abs(x - y) > tolerance;
            return isDifferent;
        }

        internal static CultureInfo GetCultureInfo(string cultureInfo, bool logException = false)
        {
            CultureInfo result = null;

            if (!string.IsNullOrEmpty(cultureInfo))
            {
                try
                {
                    result = CultureInfo.GetCultureInfo(cultureInfo);
                }
                catch (CultureNotFoundException exception)
                {
                    if (logException)
                    {
                        Logger.Instance.Log(ErrorLevel.Error, $"Culture '{cultureInfo}' not found. Message: '{exception.Message}'.");
                    }
                }
            }

            return result;
        }

    }
}