﻿using System;
using System.IO;
using System.Xml;
using System.Xml.Xsl;
using System.Collections.Generic;
using System.Text;
using Dynamicweb.Rendering;
using Dynamicweb.Ecommerce.Orders;
using System.Xml.XPath;
using Dynamicweb.Security.UserManagement;
using Dynamicweb.Environment;
using Dynamicweb.Extensibility.Notifications;
using Dynamicweb.Ecommerce.LiveIntegration.Logging;
using Dynamicweb.Ecommerce.Products;
using System.Web;
using Dynamicweb.DataIntegration.Integration.ERPIntegration;

namespace Dynamicweb.Ecommerce.LiveIntegration
{
    /// <summary>
    /// Represents sort direction
    /// </summary>
    internal enum SortDirection
    {
        Asc = 0,
        Desc
    }

    /// <summary>
    /// Static manager to handle integration customer center requests.
    /// </summary>
    internal class IntegrationCustomerCenterHandler
    {
        private static readonly string OrderLinesLoop = "OrderLinesLoop";

        /// <summary>
        /// Retrieves integration customer center item details
        /// </summary>
        /// <param name="template">Template to render item details</param>
        /// <param name="callType">Item type(OpenOrder,Invoice,Credit)</param>
        /// <param name="user">User</param>
        /// <param name="itemID">item id</param>
        /// <returns>Template with rendered item details</returns>
        internal static Template RetrieveItemDetailsFromRemoteSystem(Template template, string callType, User user, string itemID)
        {
            if (!string.IsNullOrEmpty(itemID) && !string.IsNullOrEmpty(Context.Current.Request.GetString("Redirect")) && Context.Current.Request.GetBoolean("addToCart"))
            {
                HandleReorder(itemID);
            }
            else
            {
                if (!string.IsNullOrEmpty(callType) && user != null)
                {
                    string request = GetItemDetailsRequest(callType, user.CustomerNumber, itemID);
                    Logger.Instance.Log(ErrorLevel.DebugInfo, string.Format("Request RetrieveItemDetailsFromRemoteSystem sent: '{0}'.", request));
                    XmlDocument response = Connector.RetrieveDataFromRequestString(request);
                    if (response != null && !string.IsNullOrEmpty(response.InnerXml))
                    {
                        Logger.Instance.Log(ErrorLevel.DebugInfo, string.Format("Response RetrieveItemDetailsFromRemoteSystem received: '{0}'.", response.InnerXml));
                        if (template.TemplateBaseType != Template.TemplateType.Xslt)
                        {
                            ProcessItemDetailsResponse(response, template, callType);
                        }
                        else
                        {
                            MakeXslTransformation(response, template);
                        }
                    }
                    else
                    {
                        Logger.Instance.Log(ErrorLevel.ResponseError, "Response RetrieveItemDetailsFromRemoteSystem returned null.");
                    }
                }
            }
            return template;
        }

        /// <summary>
        /// Retrieves integration customer center items list
        /// </summary>
        /// <param name="template">Template to render items list</param>
        /// <param name="callType">Item type(OpenOrder,Invoice,Credit)</param>
        /// <param name="user">User</param>
        /// <param name="pageSize">List page size</param>
        /// <param name="pageIndex">List page index</param>
        /// <param name="totalItemsCount">Items count</param>
        /// <returns>Template with rendered items list</returns>
        internal static Template RetrieveItemsListFromRemoteSystem(Template template, string callType, User user, int pageSize, int pageIndex, out int totalItemsCount)
        {
            totalItemsCount = 0;
            if (!string.IsNullOrEmpty(callType) && user != null)
            {
                string request = GetRequest(callType, true, user.CustomerNumber, null, pageSize, pageIndex);
                Logger.Instance.Log(ErrorLevel.DebugInfo, string.Format("Request RetrieveItemsListFromRemoteSystem sent: '{0}'.", request));
                XmlDocument response = Connector.RetrieveDataFromRequestString(request);
                if (response != null && !string.IsNullOrEmpty(response.InnerXml))
                {
                    Logger.Instance.Log(ErrorLevel.DebugInfo, string.Format("Response RetrieveItemsListFromRemoteSystem received: '{0}'.", response.InnerXml));
                    if (template.TemplateBaseType != Template.TemplateType.Xslt)
                    {
                        ProcessItemsListResponse(response, template, callType, pageSize, pageIndex, out totalItemsCount);
                    }
                    else
                    {
                        MakeXslTransformation(response, template);
                    }
                }
                else
                {
                    Logger.Instance.Log(ErrorLevel.ResponseError, "Response RetrieveItemsListFromRemoteSystem returned null.");
                }
            }
            return template;
        }

        private static string GetItemDetailsRequest(string callType, string userID, string itemID)
        {
            return GetRequest(callType, false, userID, itemID, 0, 0);
        }

        private static string GetRequest(string callType, bool isList, string userID, string itemID, int pageSize, int pageIndex)
        {
            string ret = string.Format("type=\"{0}\" customerID=\"{1}\"", callType, userID);
            if (isList)
            {
                int firstItem = pageSize * pageIndex - pageSize + 1;
                if (firstItem <= 0)
                    firstItem = 1;

                string sorting = string.Empty;
                if (!string.IsNullOrEmpty(Context.Current.Request.GetString("ICCSortByField")))
                {
                    SortDirection sortDirection = SortDirection.Asc;
                    Enum.TryParse<SortDirection>(Context.Current.Request.GetString("ICCSortByDirection"), out sortDirection);
                    sorting = string.Format(" sortByField=\"{0}\" sortDirection=\"{1}\"", Context.Current.Request.GetString("ICCSortByField"), sortDirection.ToString());
                }

                string search = string.Empty;
                if (!string.IsNullOrEmpty(Context.Current.Request.GetString("ICCSearchField")) && !string.IsNullOrEmpty(Context.Current.Request.GetString("ICCSearchValue")))
                {
                    search = string.Format(" searchField=\"{0}\" searchValue=\"{1}\"", Context.Current.Request.GetString("ICCSearchField"), Context.Current.Request.GetString("ICCSearchValue"));
                }

                var queryParameters = new StringBuilder();
                foreach (string key in Context.Current.Request.QueryString.AllKeys)
                {

                    if (string.IsNullOrEmpty(Context.Current.Request.GetString(key)))
                        continue;

                    if (key.ToLower().StartsWith("icc_"))
                    {
                        queryParameters.AppendFormat(" {0}=\"{1}\"", key.Substring(4), Context.Current.Request.GetString(key));
                    }
                }

                foreach (string key in Context.Current.Request.Form.AllKeys)
                {

                    if (string.IsNullOrEmpty(Context.Current.Request.GetString(key)))
                        continue;

                    if (key.ToLower().StartsWith("icc_"))
                    {
                        queryParameters.AppendFormat(" {0}=\"{1}\"", key.Substring(4), Context.Current.Request.GetString(key));
                    }
                }

                ret = string.Format("<GetList {0} requestAmount=\"{1}\" firstItem=\"{2}\"{3}{4}{5}></GetList>", ret, pageSize, firstItem, sorting, search, queryParameters.ToString());

            }
            else if (!string.IsNullOrEmpty(itemID))
            {
                ret = string.Format("<GetItem {0}  documentNO=\"{1}\"></GetItem>", ret, itemID);
            }
            return ret;
        }

        private static void ProcessItemDetailsResponse(XmlDocument response, Template template, string callType)
        {
            try
            {
                XmlNode orderNode = response.SelectSingleNode("//item [@table='EcomOrders']");
                //Set Order tags
                if (orderNode != null && orderNode.ChildNodes.Count > 0)
                {
                    foreach (XmlNode itemProperty in orderNode.ChildNodes)
                    {
                        if (itemProperty.Attributes["columnName"] != null)
                        {
                            string tagName = itemProperty.Attributes["columnName"].Value;
                            template.SetTag(tagName, itemProperty.InnerText);
                        }
                    }
                }
                //Set OrderLines loop
                XmlNodeList orderLinesNodes = response.SelectNodes("//item [@table='EcomOrderLines']");
                if (orderLinesNodes != null && orderLinesNodes.Count > 0)//Process OrderLines
                {
                    Template orderLinesLoop = template.GetLoop(OrderLinesLoop);

                    foreach (XmlNode orderLineNode in orderLinesNodes)
                    {
                        foreach (XmlNode itemProperty in orderLineNode.ChildNodes)
                        {
                            if (itemProperty.Attributes["columnName"] != null)
                            {
                                string tagName = itemProperty.Attributes["columnName"].Value;
                                orderLinesLoop.SetTag(tagName, itemProperty.InnerText);
                            }
                        }
                        orderLinesLoop.CommitLoop();
                    }
                }
                SetUrlTags(template);
            }
            catch (Exception e)
            {
                Logger.Instance.Log(ErrorLevel.Error, string.Format("Response does not match schema: '{0}'.", e.Message));
            }
        }

        private static void ProcessItemsListResponse(XmlDocument response, Template template, string callType, int pageSize, int pageIndex, out int totalItemsCount)
        {
            totalItemsCount = 0;
            string tagPrefix = string.Format("Ecom:IntegrationCustomerCenter.{0}", callType);
            try
            {
                XmlNodeList items = response.SelectNodes("//item");

                if (items != null && items.Count > 0)
                {
                    XmlNode itemsNode = response.SelectSingleNode("Items");
                    bool isTotalCountAttributePresent = itemsNode != null && itemsNode.Attributes["totalCount"] != null;
                    if (isTotalCountAttributePresent)
                    {
                        Int32.TryParse(itemsNode.Attributes["totalCount"].Value, out totalItemsCount);
                    }
                    if (totalItemsCount <= 0)
                    {
                        totalItemsCount = items.Count;
                    }

                    string itemsLoopName = string.Format("{0}Loop", tagPrefix);
                    Template itemsLoop = template.GetLoop(itemsLoopName);

                    int index = 0;
                    bool usePaging = totalItemsCount > pageSize;
                    pageIndex = pageIndex > 0 ? pageIndex - 1 : 0;
                    int startIndex = pageSize * pageIndex;
                    int endIndex = startIndex + pageSize;

                    foreach (XmlNode item in items)
                    {
                        if (index >= startIndex || !usePaging || isTotalCountAttributePresent)
                        {
                            foreach (XmlNode itemProperty in item.ChildNodes)
                            {
                                if (itemProperty.Attributes["columnName"] != null)
                                {
                                    string tagName = itemProperty.Attributes["columnName"].Value;
                                    itemsLoop.SetTag(tagName, itemProperty.InnerText);
                                }
                            }
                            SetUrlTags(itemsLoop);
                            itemsLoop.CommitLoop();
                        }
                        index++;
                        if (usePaging && index >= endIndex)
                        {
                            break;
                        }
                    }
                }
                else
                {
                    //set empty list tag
                    template.SetTag(string.Format("{0}.EmptyList", tagPrefix), true);
                }
            }
            catch (Exception e)
            {
                Logger.Instance.Log(ErrorLevel.Error, string.Format("Response does not match schema: '{0}'.", e.Message));
            }
        }

        private static void MakeXslTransformation(XmlDocument doc, Template template)
        {
            try
            {
                XslCompiledTransform xslTransform = new XslCompiledTransform();
                StringWriter writer = new StringWriter();
                xslTransform.Load(new XPathDocument(new StringReader(template.Html)));
                xslTransform.Transform(doc.CreateNavigator(), null, writer);
                template.Html = writer.ToString();
            }
            catch (Exception ex)
            {
                string msg = string.Format("Error in xsl transformation: '{0}'.", ex.Message);
                template.Html = msg;
                Logger.Instance.Log(ErrorLevel.Error, msg);
            }
        }

        private static void SetUrlTags(Template template)
        {
            var user = User.GetCurrentExtranetUser();
            string userID = user != null ? user.ID.ToString() : string.Empty;
            string orderID = Common.Context.Cart != null ? Common.Context.Cart.Id : string.Empty;
            string shopID = string.Empty;
            if (Dynamicweb.Frontend.PageView.Current() != null && Dynamicweb.Frontend.PageView.Current().Area != null)
            {
                shopID = Dynamicweb.Frontend.PageView.Current().Area.EcomShopId;
            }
            template.SetTag("UrlUserID", userID);
            template.SetTag("UrlOrderID", orderID);
            template.SetTag("UrlShopID", shopID);
        }

        private static void HandleReorder(string itemID)
        {
            OrderService orderService = new OrderService();
            Order order = orderService.GetById(itemID);
            if (order != null)
            {
                Order ecomCart = null;
                if (!Frontend.Cart.CartCatch.CartIsPresent())
                {
                    ecomCart = new Order();
                    ecomCart.IsCart = true;
                    ecomCart.LanguageId = Common.Context.LanguageID;
                    Common.Context.SetCart(ecomCart);
                }

                ecomCart = Common.Context.Cart;
                OrderLineService orderLineService = new OrderLineService();

                foreach (OrderLine orderLineToCopy in order.OrderLines)
                {
                    var OrderlineBuilder = new OrderLineBuilderConfig()
                    {
                        ProductId = orderLineToCopy.ProductId,
                        Quantity = orderLineToCopy.Quantity,
                        VariantId = orderLineToCopy.ProductVariantId,
                        VariantText = orderLineToCopy.ProductVariantText,
                        UnitId = orderLineToCopy.UnitId,
                        OrderLineType = orderLineToCopy.OrderLineType,
                        ReferenceUrl = orderLineToCopy.Reference,
                        PageId = orderLineToCopy.PageId,
                        ProductName = string.IsNullOrEmpty(orderLineToCopy.ProductId) ? orderLineToCopy.ProductName : string.Empty,
                        ProductNumber = string.IsNullOrEmpty(orderLineToCopy.ProductId) ? orderLineToCopy.ProductNumber : string.Empty,
                        OrderLineFieldValues = orderLineToCopy.OrderLineFieldValues,
                    };

                    var newOrderLine = orderLineService.OrderLineBuilder(ecomCart, OrderlineBuilder);

                    Frontend.Cart.CartCatch.SaveCart();

                    //Prepare Orderline prices.                    
                    Dictionary<Product, double> prods = new Dictionary<Product, double>();
                    foreach (OrderLine ol in ecomCart.OrderLines)
                    {
                        if (ol.HasType(OrderLineType.Product))
                        {
                            if (!prods.ContainsKey(ol.Product))
                            {
                                prods.Add(ol.Product, ol.Quantity);
                            }
                        }
                    }
                    Prices.PriceManager.PreparePrices(prods);

                    NotificationManager.Notify(Ecommerce.Notifications.Ecommerce.Cart.Line.Added, 
                        new Ecommerce.Notifications.Ecommerce.Cart.Line.AddedArgs(newOrderLine, ecomCart));
                }
                ecomCart.Comment = order.Comment;
                ecomCart.CustomerComment = order.CustomerComment;
                OrderFieldValueCollection orderFieldValues = new OrderFieldValueCollection();
                foreach (OrderFieldValue orderField in order.OrderFieldValues)
                {
                    orderFieldValues.Add(new OrderFieldValue(orderField.OrderField, orderField.Value));
                }
                ecomCart.OrderFieldValues = orderFieldValues;                
                orderService.Save(ecomCart);
                orderService.RemoveNoneActiveProducts(ecomCart);                

                string redirectUrl = Context.Current.Request.GetString("Redirect");
                if (Context.Current.Request.GetInt32("Redirect") != 0)
                {
                    redirectUrl = "/Default.aspx?ID=" + Context.Current.Request.GetString("Redirect");
                }
                Context.Current.Response.Redirect(redirectUrl);
                Context.Current.Response.End();
            }
            else
            {
                Logger.Instance.Log(ErrorLevel.Error, string.Format("Can not find order with ID = '{0}'", itemID));
            }
        }

        /// <summary>
        /// Retrieves order information in pdf
        /// </summary>
        /// <param name="request">Request</param>
        /// <param name="response">Response</param>
        /// <param name="securityKey">Security key</param>
        internal static void RetrievePDF(HttpRequest request, HttpResponse response, string securityKey)
        {
            string id = request["id"];
            string type = request["type"];
            bool forceDownload = Context.Current.Request.GetBoolean("forceDownload");

            string requestString = "<GetPDFForItem type=\"" + type + "\" id=\"" + id + "\"";
            User user = User.GetCurrentExtranetUser();
            if (user != null)
            {
                requestString += " externalUserID=\"" + user.CustomerNumber + "\"";
            }
            requestString += "></GetPDFForItem>";

            string base64EncodedPDF = string.Empty;
            try
            {
                Logger.Instance.Log(ErrorLevel.DebugInfo, string.Format("Request RetrievePDF sent: '{0}'.", requestString));
                base64EncodedPDF = ErpServiceCaller.GetDataFromRequestString(UrlHandler.Instance.GetWebServiceUrl(), securityKey, requestString);
                Logger.Instance.Log(ErrorLevel.DebugInfo, string.Format("Response RetrievePDF received: '{0}'.", base64EncodedPDF));
            }
            catch (Exception ex)
            {
                Logger.Instance.Log(ErrorLevel.ResponseError, string.Format("Response RetrievePDF returned error: '{0}'.", ex.Message));
                throw ex;
            }

            MemoryStream inputStream = new MemoryStream();
            StreamWriter writer = new StreamWriter(inputStream);
            writer.Write(base64EncodedPDF);
            writer.Flush();
            inputStream.Position = 0;
            response.ContentType = "application/pdf";

            if (forceDownload)
            {
                string fileName = string.Format("IntegrationCustomerCenterItem{0}.pdf", id);
                string filePath = HttpContext.Current.Server.MapPath("/" + Dynamicweb.Content.Files.FilesAndFolders.GetFilesFolderName() + "/System/Log/LiveIntegration/" + fileName);
                using (Stream stream = File.OpenWrite(filePath))
                {
                    DecodeStream(inputStream, stream);
                }
                response.Clear();
                response.AddHeader("content-disposition", string.Format("attachment;filename={0}", fileName));
                response.WriteFile(filePath);
                response.Flush();
                File.Delete(filePath);
            }
            else
            {
                DecodeStream(inputStream, response.OutputStream);
                response.OutputStream.Flush();
                response.OutputStream.Close();
            }
        }

        /// <summary>
        /// Decodes the stream.
        /// </summary>
        /// <param name="inStream">The in stream.</param>
        /// <param name="output">The output.</param>
        private static void DecodeStream(Stream inStream, Stream output)
        {
            using (System.Security.Cryptography.ICryptoTransform transform = new System.Security.Cryptography.FromBase64Transform())
            {
                using (var cryptStream = new System.Security.Cryptography.CryptoStream(inStream, transform, System.Security.Cryptography.CryptoStreamMode.Read))
                {
                    byte[] buffer = new byte[4096];
                    int bytesRead = cryptStream.Read(buffer, 0, buffer.Length);
                    while (bytesRead > 0)
                    {
                        output.Write(buffer, 0, bytesRead);
                        bytesRead = cryptStream.Read(buffer, 0, buffer.Length);
                    }
                }
            }
        }

    }
}
