﻿using Dynamicweb.Core.Helpers;
using Dynamicweb.Ecommerce.LiveIntegration.Configuration;
using Dynamicweb.Mailing;
using Dynamicweb.Rendering;
using System;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Threading;
using Constants = Dynamicweb.Ecommerce.LiveIntegration.Configuration.Constants;

namespace Dynamicweb.Ecommerce.LiveIntegration.Logging
{
    /// <summary>
    /// Helper class for logging.
    /// </summary>
    public class Logger
    {
        /// <summary>
        /// The culture information.
        /// </summary>
        private static readonly CultureInfo CultureInfo = new CultureInfo("en-US");

        /// <summary>
        /// The synchronize lock.
        /// </summary>
        private static readonly object SyncLock = new object();

        /// <summary>
        /// The log file
        /// </summary>
        private string _logFile;

        /// <summary>
        /// Prevents a default instance of the <see cref="Logger"/> class from being created.
        /// </summary>
        private Logger()
        {
            if (string.IsNullOrEmpty(_logFile))
            {
                _logFile = Core.SystemInformation.MapPath("/Files/System/Log/LiveIntegration/" + Constants.AddInName + ".log");
            }
        }

        /// <summary>
        /// Gets the instance.
        /// </summary>
        /// <value>The instance.</value>
        public static Logger Instance { get; } = new Logger();

        /// <summary>
        /// Gets or sets a value that determines if connection errors should be logged.
        /// </summary>
        /// <value><c>true</c> if [log connection errors]; otherwise, <c>false</c>.</value>
        private bool LogConnectionErrors => Settings.Instance.LogConnectionErrors;

        /// <summary>
        /// Gets or sets a value that determines if debug info should be logged.
        /// </summary>
        /// <value><c>true</c> if [log debug information]; otherwise, <c>false</c>.</value>
        private bool LogDebugInfo => Settings.Instance.LogDebugInfo;

        /// <summary>
        /// Gets or sets a value that determines if general info should be logged.
        /// </summary>
        /// <value><c>true</c> if [log general errors]; otherwise, <c>false</c>.</value>
        private bool LogGeneralErrors => Settings.Instance.LogGeneralErrors;

        /// <summary>
        /// Gets or sets a value that determines if response errors should be logged.
        /// </summary>
        /// <value><c>true</c> if [log response errors]; otherwise, <c>false</c>.</value>
        private bool LogResponseErrors => Settings.Instance.LogResponseErrors;

        /// <summary>
        /// Sends an mail with error information according to configuration.
        /// </summary>
        /// <param name="message">The error/success message to send.</param>
        /// <returns><c>true</c> if email was sent, <c>false</c> otherwise.</returns>
        public static bool SendMail(string message)
        {
            bool result = false;
            string notificationTemplate = Settings.Instance.NotificationTemplate;
            string notificationEmail = Settings.Instance.NotificationEmail;

            if (!string.IsNullOrEmpty(message) && !string.IsNullOrEmpty(notificationTemplate) && StringHelper.IsValidEmailAddress(notificationEmail))
            {
                Template templateInstance = new Template(notificationTemplate);
                templateInstance.SetTag("Ecom:LiveIntegration.AddInName", Constants.AddInName);
                templateInstance.SetTag("Ecom:LiveIntegration.ErrorMessage", message);

                string notificationEmailSubject = Settings.Instance.NotificationEmailSubject;
                string notificationEmailSenderEmail = Settings.Instance.NotificationEmailSenderEmail;
                string notificationEmailSenderName = Settings.Instance.NotificationEmailSenderName;

                using (var mail = new System.Net.Mail.MailMessage())
                {
                    mail.IsBodyHtml = true;
                    mail.Subject = notificationEmailSubject;
                    mail.SubjectEncoding = System.Text.Encoding.UTF8;
                    mail.From = new System.Net.Mail.MailAddress(notificationEmailSenderEmail, notificationEmailSenderName, System.Text.Encoding.UTF8);

                    // Set parameters for MailMessage
                    mail.To.Add(notificationEmail);
                    mail.BodyEncoding = System.Text.Encoding.UTF8;

                    // Render Template and set as Body
                    mail.Body = templateInstance.Output();

                    // Send mail
                    EmailHandler.Send(mail);
                }

                result = true;
            }

            return result;
        }        

        /// <summary>
        /// Gets the error messages sine the last email was sent.
        /// </summary>
        /// <returns>Log data</returns>
        public string GetLastLogData()
        {
            string result = string.Empty;

            lock (SyncLock)
            {
                foreach (var line in File.ReadLines(_logFile, System.Text.Encoding.UTF8).Reverse())
                {
                    if (line.Contains(ErrorLevel.DebugInfo.ToString()))
                    {
                        // ignore debug info
                    }
                    else if (!line.Contains(ErrorLevel.EmailSend.ToString()))
                    {
                        result += line + "<br>";
                    }
                    else
                    {
                        break;
                    }
                }
            }

            return result;
        }

        /// <summary>
        /// Logs at the specified error level.
        /// </summary>
        /// <param name="errorLevel">The error level.</param>
        /// <param name="logline">The log line.</param>
        public void Log(ErrorLevel errorLevel, string logline)
        {
            if (!string.IsNullOrEmpty(_logFile))
            {
                bool isAllowedAddToLog = IsAllowedAddToLog(errorLevel);
                lock (SyncLock)
                {
                    if (isAllowedAddToLog)
                    {
                        KeepOrTruncateFile();
                        logline = $"{DateTime.Now.ToString(CultureInfo)}: {errorLevel}: {logline}{System.Environment.NewLine}";

                        try
                        {
                            TextFileHelper.WriteTextFile(logline, _logFile, true, System.Text.Encoding.UTF8);
                        }
                        catch (IOException)
                        {
                            Thread.Sleep(500);
                            TextFileHelper.WriteTextFile(logline, _logFile, true, System.Text.Encoding.UTF8);
                        }
                        catch
                        {
                            var fileName = Context.Current.Server.MapPath(
                                $"/Files/System/Log/LiveIntegration/{Guid.NewGuid()}.log");
                            TextFileHelper.WriteTextFile(logline, fileName, false, System.Text.Encoding.UTF8);
                        }
                    }
                }

                if (errorLevel != ErrorLevel.DebugInfo && errorLevel != ErrorLevel.EmailSend)
                {
                    SendLog(logline, isAllowedAddToLog);
                }
            }
        }

        /// <summary>
        /// Determines whether the configuration is set to log at the specified log level.
        /// </summary>
        /// <param name="level">The level to check.</param>
        /// <returns><c>true</c> if [is allowed add to log]; otherwise, <c>false</c>.</returns>
        private bool IsAllowedAddToLog(ErrorLevel level)
        {
            bool result = false;
            switch (level)
            {
                case ErrorLevel.ConnectionError:
                    result = LogConnectionErrors;
                    break;

                case ErrorLevel.ResponseError:
                    result = LogResponseErrors;
                    break;

                case ErrorLevel.Error:
                    result = LogGeneralErrors;
                    break;

                case ErrorLevel.DebugInfo:
                    result = LogDebugInfo;
                    break;

                case ErrorLevel.EmailSend:
                    result = true;
                    break;
            }

            return result;
        }

        /// <summary>
        /// Rolls over the log file and creates a new one or truncates the existing file, depending on the settings.
        /// </summary>
        private void KeepOrTruncateFile()
        {
            FileInfo fi = new FileInfo(_logFile);
            int maxSize = Settings.Instance.LogMaxSize;
            if (maxSize > 100)
            {
                maxSize = 100;
            }

            if (!fi.Exists || fi.Length < maxSize * 1024 * 1024)
            {
                return;
            }

            if (Settings.Instance.KeepLogFiles)
            {
                MoveToHistoryFile(fi);
            }
            else
            {
                TruncateLogFileFileInfo(fi, maxSize);
            }
        }

        /// <summary>
        /// Moves to history file.
        /// </summary>
        /// <param name="fi">The fi.</param>
        private void MoveToHistoryFile(FileInfo fi)
        {
            try
            {
                var fileName = Path.GetFileNameWithoutExtension(_logFile);
                var newFileName = $"{fileName}-{DateTime.Now:yyyyMMddHHmmss}{fi.Extension}";
                var folder = fi.DirectoryName;

                if (!string.IsNullOrEmpty(folder))
                {
                    var newLocation = Path.Combine(folder, newFileName);
                    File.Move(_logFile, newLocation);
                }
            }
            catch
            {
                var logLine = $"{DateTime.Now.ToString(CultureInfo)}: {ErrorLevel.Error}: Cannot move log file.{System.Environment.NewLine}";
                TextFileHelper.WriteTextFile(logLine, _logFile, true, System.Text.Encoding.UTF8);
            }
        }

        /// <summary>
        /// Sends the log.
        /// </summary>
        /// <param name="lastError">The last error.</param>
        /// <param name="isLastErrorInLog">if set to <c>true</c> [is last error in log].</param>
        private void SendLog(string lastError, bool isLastErrorInLog)
        {
            string frequencySettings = Settings.Instance.NotificationSendingFrequency;
            if (string.IsNullOrEmpty(frequencySettings))
            {
                return;
            }

            var frequency = Helpers.GetEnumValueFromString(frequencySettings, NotificationFrequency.Never);
            if (frequency == NotificationFrequency.Never)
            {
                return;
            }

            // Get last time when the email was sent
            DateTime lastTimeSend = Settings.LastNotificationEmailSent;
            bool emailSent = false;
            if (lastTimeSend == DateTime.MinValue)
            {
                emailSent = SendMail(lastError);
            }
            else
            {
                // send email if the frequency interval already passed
                if (DateTime.Now.Subtract(lastTimeSend) >= TimeSpan.FromMinutes((double)frequency))
                {
                    if (!isLastErrorInLog)
                    {
                        emailSent = SendMail(GetLastLogData() + lastError);
                    }
                    else
                    {
                        emailSent = SendMail(GetLastLogData());
                    }
                }
            }

            if (emailSent)
            {
                Log(ErrorLevel.EmailSend, "Send e-mail with errors");

                // used for getting the last errors appeared for the future email
                Settings.LastNotificationEmailSent = DateTime.Now;
            }
        }

        /// <summary>
        /// Truncates the log file.
        /// </summary>
        /// <param name="fi">The fi.</param>
        /// <param name="maxSize">The maximum size.</param>
        private void TruncateLogFileFileInfo(FileInfo fi, int maxSize)
        {
            try
            {
                string folder = fi.DirectoryName;
                if (folder != null)
                {
                    string newFile = Path.Combine(folder, Path.GetRandomFileName());

                    using (FileStream ws = new FileStream(newFile, FileMode.CreateNew))
                    {
                        using (FileStream rs = new FileStream(_logFile, FileMode.Open))
                        {
                            rs.Seek((long)((fi.Length - maxSize) * 1024 * 1024 * 0.5), SeekOrigin.Begin); // -50%
                            int currChar;

                            while ((currChar = rs.ReadByte()) != -1)
                            {
                                if (currChar != '\r')
                                {
                                    continue;
                                }

                                currChar = rs.ReadByte();

                                if (currChar == -1 || currChar == '\n')
                                {
                                    break;
                                }
                            }

                            rs.CopyTo(ws, 1048576);
                        }

                        ws.Flush(true);
                    }

                    File.Delete(_logFile);
                    File.Move(newFile, _logFile);
                }
            }
            catch
            {
                string logLine = DateTime.Now.ToString(CultureInfo) + ": " + ErrorLevel.Error + ": Cannot truncate log file." + System.Environment.NewLine;
                TextFileHelper.WriteTextFile(logLine, _logFile, true, System.Text.Encoding.UTF8);
            }
        }
    }
}