﻿using Dynamicweb.Content.Files.Information;
using Dynamicweb.DataIntegration;
using Dynamicweb.Ecommerce.LiveIntegration.Configuration;
using Dynamicweb.Ecommerce.Orders;
using Dynamicweb.Ecommerce.Shops;
using Dynamicweb.Extensibility.AddIns;
using Dynamicweb.Extensibility.Editors;
using Dynamicweb.SystemTools;
using System;
using System.Collections;

namespace Dynamicweb.Ecommerce.LiveIntegration.ScheduledTasks
{    
    /// <summary>
    /// Class QueuedOrdersSyncScheduledTask.
    /// </summary>
    /// <seealso cref="BatchIntegrationScheduledTaskAddin" />
    /// <seealso cref="IDropDownOptions" />
    [AddInName("LiveIntegrationQueuedOrdersSync")]
    [AddInLabel("Sync queued orders using Live Integration")]
    [AddInDescription("Sync queued orders using Live Integration")]
    [AddInIgnore(false)]
    [AddInUseParameterGrouping(true)]
    public class QueuedOrdersSyncScheduledTask : BatchIntegrationScheduledTaskAddin, IDropDownOptions
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="QueuedOrdersSyncScheduledTask"/> class.
        /// </summary>
        public QueuedOrdersSyncScheduledTask()
        {
            MinutesCompleted = 5;
            MaxOrdersToProcess = 25;
            ShopId = string.Empty;
            ExcludeRecurrent = true;
        }

        #region Parameters

        /// <summary>
        /// Gets or sets the minutes completed.
        /// </summary>
        /// <value>The minutes completed.</value>
        [AddInParameter("Finished for X minutes")]
        [AddInParameterEditor(typeof(IntegerNumberParameterEditor), "NewUIcheckbox=true;Value=true;")]
        [AddInParameterGroup("A) Queued Orders")]
        public int MinutesCompleted { get; set; }

        /// <summary>
        /// Gets or sets the maximum orders to process.
        /// </summary>
        /// <value>The maximum orders to process.</value>
        [AddInParameter("Maximum orders to process in each execution")]
        [AddInParameterEditor(typeof(IntegerNumberParameterEditor), "NewUIcheckbox=true;Value=true;")]
        [AddInParameterGroup("A) Queued Orders")]
        public int MaxOrdersToProcess { get; set; }

        /// <summary>
        /// Gets or sets the shop identifier.
        /// </summary>
        /// <value>The shop identifier.</value>
        [AddInParameter("Shop")]
        [AddInParameterEditor(typeof(DropDownParameterEditor), "NewGUI=true;none=false")]
        [AddInParameterGroup("A) Queued Orders")]
        public string ShopId { get; set; }

        /// <summary>
        /// Gets or sets the order states.
        /// </summary>
        /// <value>The order states.</value>
        [AddInParameter("Order States")]
        [AddInParameterEditor(typeof(DropDownParameterEditor), "NewGUI=true;none=false;multiple=true")]
        [AddInParameterGroup("A) Queued Orders")]
        public string OrderStates { get; set; }

        /// <summary>
        /// Gets or sets a value indicating whether [exclude recurrent].
        /// </summary>
        /// <value><c>true</c> if [exclude recurrent]; otherwise, <c>false</c>.</value>
        [AddInParameter("Exclude recurring order templates")]
        [AddInParameterEditor(typeof(YesNoParameterEditor), "NewUIcheckbox=true;Value=true;")]
        [AddInParameterGroup("A) Queued Orders")]
        public bool ExcludeRecurrent { get; set; }

        #endregion Parameters

        /// <summary>
        /// Main method in ScheduledTask Addin - is run when scheduled Task is run
        /// </summary>
        /// <returns><c>true</c> if task run was successful, <c>false</c> otherwise.</returns>
        public override bool Run()
        {
            SetupLogging();

            bool result = false;
            string error = string.Empty;

            try
            {
                string shopFilter = string.IsNullOrEmpty(ShopId) ? "OrderShopID" : "'" + ShopId + "'";
                string recurringOrderFilter = ExcludeRecurrent ? "0" : "IsNull (OrderIsRecurringOrderTemplate, 0)";
                string orderStatesFilter = string.IsNullOrEmpty(OrderStates) ? " = OrderStateID" : "in ('" + OrderStates.Replace(",", "','") + "')";

                string sql = string.Format(
                    @"SELECT TOP {2} * FROM EcomOrders
                    WHERE OrderComplete = 1
                    AND OrderDeleted = 0
                    AND (OrderIntegrationOrderID IS NULL OR OrderIntegrationOrderID = '')
                    AND ISNULL(OrderIsExported, 0) = 0
                    AND OrderShopID = {0}
                    AND ISNULL(OrderIsRecurringOrderTemplate, 0) = {3}
                    AND OrderCompletedDate < DATEADD(MINUTE, -{1}, GETDATE())
                    AND OrderStateID {4};",
                    shopFilter,
                    MinutesCompleted,
                    MaxOrdersToProcess,
                    recurringOrderFilter,
                    orderStatesFilter);

                OrderCollection ordersToSync = Order.GetOrders(sql, true);

                foreach (var order in ordersToSync)
                {
                    if (Global.IntegrationEnabledFor(order.ShopId))
                    {
                        OrderHandler.UpdateOrder(order, SubmitType.ScheduledTask);
                    }
                }

                result = true;
            }
            catch (Exception e)
            {
                error = e.Message;
            }
            finally
            {
                if (!string.IsNullOrEmpty(error))
                {
                    // Send email with error
                    SendMail(error, MessageType.Error);
                }
                else
                {
                    // Send mail with success
                    SendMail(Translate.Translate("Scheduled task completed successfully"));
                }
            }

            WriteTaskResultToLog(result);

            return result;
        }

        /// <summary>
        /// Get options for editors with multiple values
        /// </summary>
        /// <param name="dropdownName">Name of the dropdown.</param>
        /// <returns>Hashtable.</returns>
        public Hashtable GetOptions(string dropdownName)
        {
            var options = new Hashtable();
            switch (dropdownName)
            {
                case "Shop":
                    options.Add(string.Empty, "Any");
                    var shops = Shop.GetShops();
                    foreach (var shop in shops)
                    {
                        options.Add(shop.Id, shop.Name);
                    }

                    break;

                case "Order States":
                    foreach (OrderState state in OrderState.GetAllOrderStates())
                    {
                        options.Add(state.Id, state.Name);
                    }

                    break;
            }

            return options;
        }
    }
}